unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons;

type
  TMainForm = class(TForm)
    LogMemo: TMemo;
    Panel2: TPanel;
    CardCombo: TComboBox;
    ControlPanel: TPanel;
    WDGTime: TTimer;
    TempLabel: TLabel;
    TempButton: TButton;
    DIGroup: TGroupBox;
    DILabel0: TLabel;
    DILabel1: TLabel;
    DILabel2: TLabel;
    DILabel3: TLabel;
    DILabel4: TLabel;
    DILabel5: TLabel;
    PollTime: TTimer;
    DOGroup: TGroupBox;
    DOCheck0: TCheckBox;
    DOCheck4: TCheckBox;
    DOCheck3: TCheckBox;
    DOCheck1: TCheckBox;
    DOCheck2: TCheckBox;
    DOWriteButton: TButton;
    AIGroup: TGroupBox;
    InternalTempLabel: TLabel;
    RHLabel: TLabel;
    ADCLabel0: TLabel;
    ADCLabel1: TLabel;
    ADCLabel3: TLabel;
    ADCLabel2: TLabel;
    PWMGroup: TGroupBox;
    PWMButton0: TButton;
    PWMButton1: TButton;
    PWMButton2: TButton;
    PWMButton3: TButton;
    PWMButton4: TButton;
    PWMStopButton0: TBitBtn;
    PWMStopButton1: TBitBtn;
    PWMStopButton2: TBitBtn;
    PWMStopButton3: TBitBtn;
    PWMStopButton4: TBitBtn;
    SpeakerGroup: TGroupBox;
    SpeakerOffButton: TButton;
    SpeakerFEFFButton: TButton;
    Speaker2kButton: TButton;
    Speaker100Button: TButton;
    DICheck: TCheckBox;
    ADCCheck: TCheckBox;
    WatchdogGroup: TGroupBox;
    WDGEventButton: TButton;
    SoftRestartRadio: TRadioButton;
    MostlySoftRestartRadio: TRadioButton;
    DisableWDGCheck: TCheckBox;
    NoRestartRadio: TRadioButton;
    GoWDGButton: TButton;
    CancelWDGButton: TButton;
    AbortButton: TButton;
    Label1: TLabel;
    DutyCycleTrack: TTrackBar;
    Label2: TLabel;
    GroupBox1: TGroupBox;
    WCRRCheck: TCheckBox;
    TempGoodCheck: TCheckBox;
    StatusDICheck1: TCheckBox;
    StatusDICheck0: TCheckBox;
    FanGoodCheck: TCheckBox;
    PowerOverVCheck: TCheckBox;
    PowerUnderVCheck: TCheckBox;
    IRQGeneratedCheck: TCheckBox;
    StatusButton: TButton;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure AbortButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure WDGEventButtonClick(Sender: TObject);
    procedure GoWDGButtonClick(Sender: TObject);
    procedure CancelWDGButtonClick(Sender: TObject);
    procedure TempButtonClick(Sender: TObject);
    procedure StatusButtonClick(Sender: TObject);
    procedure CardComboChange(Sender: TObject);
    procedure WDGPetTimer(Sender: TObject);
    procedure WDGWaitTimer(Sender: TObject);
    procedure PollTimeTimer(Sender: TObject);
    procedure DOWriteButtonClick(Sender: TObject);
    procedure SpeakerButtonClick(Sender: TObject);
    procedure PWMButtonClick(Sender: TObject);
    procedure PWMStopButtonClick(Sender: TObject);
    procedure DICheckClick(Sender: TObject);
    procedure ADCCheckClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    procedure UpdateOutputStates;
  public
    bInitTime: Boolean;
    procedure AddLine(CardNum: Integer; Line: String);
    procedure PostLogMemo;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  AIOWDG;

type
  TCardData = record
    Log: String;
    WDGPets: LongWord;
  end;

var
  CardData: array of TCardData;

procedure TMainForm.FormCreate(Sender: TObject);
var
  NumCards, CardNum: Integer;
  DeviceID: LongWord;
  NameSize: LongWord;
  Name: String;
  ErrorCode: LongWord;

begin
  bInitTime := True;
  CardCombo.Items.Clear;
  NumCards := GetNumCards;
  if NumCards = 0 then begin
    ControlPanel.Visible := False;

    LogMemo.Text := 'No cards found. The cards may not be installed, or they may be installed using a driver other than AIOWDG.sys.';
  end
  else begin
    SetLength(CardData, NumCards);
    for CardNum := 0 to NumCards - 1 do begin
      ErrorCode := QueryCardID(CardNum, @DeviceID);
      if ErrorCode <> ERROR_SUCCESS then begin
        CardCombo.Items.Add('Error ' + IntToStr(ErrorCode) + ' getting data: Card ' + IntToStr(CardNum));
        Continue;
      end;

      case DeviceID of
        $8080, $8081: begin //PCIe-WDG-CSM, PCIe-WDG-CSMA
          //WDGInit(CardNum);
        end;
      end;

      NameSize := 256;
      SetString(Name, nil, NameSize);
      ErrorCode := QueryCardName(CardNum, @NameSize, PChar(Name));
      if ErrorCode <> ERROR_SUCCESS then begin
        CardCombo.Items.Add('Error ' + IntToStr(ErrorCode) + ' getting data: Card ' + IntToStr(CardNum));
        Continue;
      end;

      SetLength(Name, NameSize);
      CardCombo.Items.Add(Name);

      CardData[CardNum].Log :=
          'AIOWDG provides special functions for watchdog cards.'#13#10#13#10
        + 'Click "Handle Watchdog IRQ" to handle the next IRQ for the card selected above with the selected action.'#13#10#13#10
        + 'You can also click "Test Watchdog Timing" to set the watchdog timer, pet it a few times, then let it time out, '
        + '"Read Temp" to read the temperature(if installed), '
        + '"Read Status" to read the card''s status byte, '
        + '"Read Digital In" to start/stop reading the card''s various digital inputs, '
        + 'or "Write Digital Out" to write to the card''s various digital outputs.'#13#10#13#10
      ;
    end;
    CardCombo.ItemIndex := 0;
    CardComboChange(CardCombo);
  end;
  bInitTime := False;
end;


procedure TMainForm.UpdateOutputStates;
var
  DOData: Byte;

begin
  DO_ReadbackAll(CardCombo.ItemIndex, @DOData);

  DOCheck0.Checked := (DOData and (1 shl DOCheck0.Tag)) <> 0;
  DOCheck1.Checked := (DOData and (1 shl DOCheck1.Tag)) <> 0;
  DOCheck2.Checked := (DOData and (1 shl DOCheck2.Tag)) <> 0;
  DOCheck3.Checked := (DOData and (1 shl DOCheck3.Tag)) <> 0;
  DOCheck4.Checked := (DOData and (1 shl DOCheck4.Tag)) <> 0;
end;


procedure TMainForm.CardComboChange(Sender: TObject);
begin
  PostLogMemo;

  DILabel0.Caption := '';
  DILabel1.Caption := '';
  DILabel2.Caption := '';
  DILabel3.Caption := '';
  DILabel4.Caption := '';
  DILabel5.Caption := '';

  InternalTempLabel.Caption := '';
  RHLabel.Caption := '';
  ADCLabel0.Caption := '';
  ADCLabel1.Caption := '';
  ADCLabel2.Caption := '';
  ADCLabel3.Caption := '';

  WCRRCheck.State := cbGrayed;
  TempGoodCheck.State := cbGrayed;
  StatusDICheck1.State := cbGrayed;
  StatusDICheck0.State := cbGrayed;
  FanGoodCheck.State := cbGrayed;
  PowerOverVCheck.State := cbGrayed;
  PowerUnderVCheck.State := cbGrayed;
  IRQGeneratedCheck.State := cbGrayed;

  TempLabel.Caption := '';
  UpdateOutputStates;
  PollTime.Enabled := True;
end;

procedure TMainForm.AbortButtonClick(Sender: TObject);
var
  CardNum: Integer;

begin
  CardNum := CardCombo.ItemIndex;
  if WDG_AbortEventRequest(CardNum) = ERROR_SUCCESS then
    AddLine(CardNum, 'Aborting IRQ detection.')
  else
    AddLine(CardNum, 'No IRQ request to abort.')
  ;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  CardNum: Integer;

begin
  {
    Abort all pending IRQ requests. If we don't do this before exiting, later
    when an IRQ comes in or somebody else calls AbortRequest the pending IRQ
    request will attempt to unlock a thread that no longer exists, which is bad.

    (We can AbortRequest for a card with no pending requests, it's easier than
    remembering which cards have pending requests.)
  }
  for CardNum := 0 to GetNumCards - 1 do
    WDG_AbortEventRequest(CardNum)
  ;
end;

procedure TMainForm.WDGEventButtonClick(Sender: TObject);
var
  Action: LongWord;
  CardNum: Integer;

begin
  CardNum := CardCombo.ItemIndex;

  if SoftRestartRadio.Checked then
    Action := WDG_ACTION_SOFT_RESTART
  else if MostlySoftRestartRadio.Checked then
    Action := WDG_ACTION_MOSTLY_SOFT_RESTART
  else
    Action := WDG_ACTION_IGNORE
  ;

  if DisableWDGCheck.Checked then
    Action := Action + WDG_ACTION_DISABLE
  ;

  WDG_HandleEvent(CardNum, Action);

  AddLine(CardNum, 'Watchdog event handler started, press "Abort IRQ Detect" to cancel.');
end;

procedure TMainForm.GoWDGButtonClick(Sender: TObject);
var
  CardNum: Integer;
  Milliseconds: Double;

begin
  CardNum := CardCombo.ItemIndex;

  Milliseconds := 4 * 1000; // 2 sec timeout
  AIOWDG_GenericVendorWrite(CardNum, $40, Round(Milliseconds), $0055, 0, nil);    // Set Watchdog Timeout, In Milliseconds

  Milliseconds := 1; // infinite reset
  WDG_SetResetDuration(CardNum, Milliseconds);

  WDG_Start(CardNum);

  AddLine(CardNum, 'Watchdog timing started... @ ' + TimeToStr(Now));
  AddLine(CardNum, 'The card will show a Solid-Green LED to indicate the firmware is ready. Each time the watchdog receives a "Pet" the Green LED will flash Cyan. '
                 + 'The Watchdog Early Warning of Imminent Timeout "Growl" will cause the Solid-Green LED to turn Solid-Amber. The actual Watchdog Timeout "Bark" will turn the LED Solid-Red.'
  );
  CardData[CardNum].WDGPets := 0;
  WDGTime.Tag := CardNum;
  WDGTime.OnTimer := WDGPetTimer;
  WDGTime.Interval := 1000;
  WDGTime.Enabled := True;
end;

procedure TMainForm.WDGPetTimer(Sender: TObject);
var
  CardNum: Integer;
  StatusByte: LongWord;

begin
  CardNum := WDGTime.Tag;

  WDG_ReadStatus(CardNum, StatusByte);
  if (StatusByte and $01) = 0 then begin
    AddLine(CardNum, 'Watchdog timed out early!');
    WDGTime.Enabled := False;
    Exit;
  end;

  Inc(CardData[CardNum].WDGPets);
  if CardData[CardNum].WDGPets > 5 then begin
    AddLine(CardNum, 'Waiting for timeout...');
    WDGTime.OnTimer := WDGWaitTimer;
    WDGTime.Interval := 100;
  end
  else begin
    WDG_Pet(CardNum);
    AddLine(CardNum, 'Petting watchdog, pet ' + IntToStr(CardData[CardNum].WDGPets) + ' of 5... @'+TimeToStr(Now));
  end;
end;

procedure TMainForm.WDGWaitTimer(Sender: TObject);
var
  CardNum: Integer;
  StatusByte: LongWord;

begin
  CardNum := WDGTime.Tag;

  WDG_ReadStatus(CardNum, StatusByte);
  if (StatusByte and $01) = 0 then begin
    AddLine(CardNum, 'Watchdog timed out... @ '+ TimeToStr(Now));
    WDGTime.Enabled := False;
  end;
end;

procedure TMainForm.CancelWDGButtonClick(Sender: TObject);
var
  CardNum: Integer;
begin
  CardNum := CardCombo.ItemIndex;
  if WDGTime.Enabled = True then
  begin
    WDG_Stop(CardNum);
    WDGTime.Enabled := False;
    AddLine(CardNum, 'Watchdog timing cancelled.');
  end else
  begin
    AddLine(CardNum, 'Watchdog timing was not running.');
  end;
end;

procedure TMainForm.TempButtonClick(Sender: TObject);
var
  TempF: Double;
  Err: LongWord;

begin
  TempF := -1;
  Err := WDG_ReadTemp(CardCombo.ItemIndex, TempF);
  if Err = ERROR_SUCCESS then
    TempLabel.Caption := Format('%.1f', [ TempF ])
  else
    TempLabel.Caption := 'Error ' + IntToStr(Err) + ' from WDGReadTemp.'
  ;
end;

procedure TMainForm.StatusButtonClick(Sender: TObject);
var
  StatusByte: LongWord;
  Err: LongWord;

begin
  Err := WDG_ReadStatus(CardCombo.ItemIndex, StatusByte);
  if Err <> ERROR_SUCCESS then begin
    StatusButton.Caption := 'Error ' + IntToStr(Err) + ' from WDGReadStatus.';
    WCRRCheck.State := cbGrayed;
    TempGoodCheck.State := cbGrayed;
    StatusDICheck1.State := cbGrayed;
    StatusDICheck0.State := cbGrayed;
    FanGoodCheck.State := cbGrayed;
    PowerOverVCheck.State := cbGrayed;
    PowerUnderVCheck.State := cbGrayed;
    IRQGeneratedCheck.State := cbGrayed;
    Exit;
  end;

  StatusButton.Caption := 'Read Status';
  WCRRCheck.Checked         := (StatusByte and (1 shl 0)) = 0;
  TempGoodCheck.Checked     := (StatusByte and (1 shl 1)) <> 0;
  StatusDICheck1.Checked    := (StatusByte and (1 shl 2)) <> 0;
  StatusDICheck0.Checked    := (StatusByte and (1 shl 3)) <> 0;
  FanGoodCheck.Checked      := (StatusByte and (1 shl 4)) <> 0;
  PowerOverVCheck.Checked   := (StatusByte and (1 shl 5)) = 0;
  PowerUnderVCheck.Checked  := (StatusByte and (1 shl 6)) = 0;
  IRQGeneratedCheck.Checked := (StatusByte and (1 shl 7)) <> 0;

  {
      BD0 Watchdog counter refresh reminder (Active low)
      BD1 Temperature good (Active high)
      BD2 Isolated Input #1 status (Same as input)
      BD3 Isolated Input #0 status (Same as input)
      BD4 Fan good (Active high)
      BD5 Power Supply overvoltage (Active low)
      BD6 Power Supply undervoltage (Active low)
      BD7 IRQ generated (Active high)
  }
end;

procedure TMainForm.AddLine(CardNum: Integer; Line: String);
begin
  CardData[CardNum].Log := CardData[CardNum].Log + Line + #13#10#13#10;
  if CardNum = CardCombo.ItemIndex then PostLogMemo;
end;

procedure TMainForm.PostLogMemo;
begin
  LogMemo.Text := CardData[CardCombo.ItemIndex].Log;
  LogMemo.SelStart := Length(CardData[CardCombo.ItemIndex].Log);
  LogMemo.SelLength := 1;
end;

procedure TMainForm.DICheckClick(Sender: TObject);
begin
  DILabel0.Caption := '';
  DILabel1.Caption := '';
  DILabel2.Caption := '';
  DILabel3.Caption := '';
  DILabel4.Caption := '';
  DILabel5.Caption := '';
end;

procedure TMainForm.ADCCheckClick(Sender: TObject);
begin
  InternalTempLabel.Caption := '';
  RHLabel.Caption := '';
  ADCLabel0.Caption := '';
  ADCLabel1.Caption := '';
  ADCLabel2.Caption := '';
  ADCLabel3.Caption := '';

  if ADCCheck.Checked then
    AI_Enable(CardCombo.ItemIndex)
  else
    AI_Disable(CardCombo.ItemIndex)
  ;
end;

procedure TMainForm.PollTimeTimer(Sender: TObject);

  function IntToBitDesc(I: Integer): AnsiString;
  begin
    if I = 0 then
      Result := 'low'
    else
      Result := 'HIGH'
    ;
  end;

  function Si7013_TempToCelsius(x: word): double;
  begin
      Result := (175.72*x)/65536.0 - 46.85;
  end;
  function Si7013_HumidityToRH(x: word) : double;
  begin
      Result :=  125.0*x/65536.0 - 6.0;
  //    Result :=  min( max((125.0*x)/65536.0 - 6.0, 0), 100 );
  end;

  function Si7013_CountsToMilliVolts(x: word) : double;
  begin
      Result :=  1.25 * x / 32.767;
  end;

var
  DIData: Byte;
  AIBuf: array of Double;

begin
  UpdateOutputStates;

  // Digital Inputs
  if DICheck.Checked then try
    DI_ReadAll(CardCombo.ItemIndex, @DIData);

    DILabel0.Caption := 'Isolated In 0: ' + IntToBitDesc(DIData and (1 shl 0));
    DILabel1.Caption := 'Isolated In 1: ' + IntToBitDesc(DIData and (1 shl 1));
    DILabel2.Caption := 'Isolated In 2: ' + IntToBitDesc(DIData and (1 shl 2));
    DILabel3.Caption := 'Isolated In 3: ' + IntToBitDesc(DIData and (1 shl 3));
    DILabel4.Caption := 'TTL In 0: '      + IntToBitDesc(DIData and (1 shl 4));
    DILabel5.Caption := 'TTL In 1: '      + IntToBitDesc(DIData and (1 shl 5));
  except
    DICheck.Checked := False;
    raise;
  end;

  // Analog Inputs
  if ADCCheck.Checked then try
   begin
      SetLength(AIBuf, 6);
      AI_ReadAll(CardCombo.ItemIndex, @AIBuf[0]);
      InternalTempLabel.Caption := 'Internal Temp: ' + Format('%.1f', [ AIBuf[0] ]) + 'C';  // internal temp
      RHLabel.Caption           := 'Humidity: ' + Format('%.1f', [ AIBuf[1] ]) + ' %RH';  // relative humidity
      ADCLabel0.Caption         := '12V Power: ' + Format('%.1f', [ AIBuf[2] ]) +  ' mV';  // 12V
      ADCLabel1.Caption         := '3.3V Power: ' + Format('%.1f', [ AIBuf[3] ]) + ' mV';  // 3.3V
// Note: The conversion from mV to degrees C/F is TBD
//     ADCLabel2.Caption         := 'External Temp: ' + Format('%.1f', [ (AIBuf[4] - 460) * 70 / 320 ]) + ' F';  // ext temp
      ADCLabel2.Caption         := 'External Temp: ' + Format('%.1f', [ AIBuf[4] - 460]) + ' mV';  // ext temp
      ADCLabel3.Caption         := 'Light Sensor: ' + Format('%.1f', [ AIBuf[5] ]) + ' mV';  // light sensor
    end;
  except
    ADCCheck.Checked := False;
    raise;
  end;
end;

procedure TMainForm.DOWriteButtonClick(Sender: TObject);
var
  DOData: Byte;

begin
  if bInitTime then Exit;
  
  DOData := 0;
  if DOCheck0.Checked then DOData := DOData or (1 shl 0);
  if DOCheck1.Checked then DOData := DOData or (1 shl 1);
  if DOCheck2.Checked then DOData := DOData or (1 shl 2);
  if DOCheck3.Checked then DOData := DOData or (1 shl 3);
  if DOCheck4.Checked then DOData := DOData or (1 shl 4);

  DO_WriteAll(CardCombo.ItemIndex, DOData);
end;
              
procedure TMainForm.SpeakerButtonClick(Sender: TObject);
begin
  Speaker_SetFreq(CardCombo.ItemIndex, (Sender as TButton).Tag);
end;

procedure TMainForm.PWMButtonClick(Sender: TObject);
var
  BitIndex: Longword;
  DutyCycle: Byte;

begin
  BitIndex := (Sender as TButton).Tag;
  DutyCycle := DutyCycleTrack.Position;
  //RepCount=-1 means PWM goes forever. Change RepCount to >0 for that many modulated pulses.
  DO_SetPWM(CardCombo.ItemIndex, BitIndex, DutyCycle, -1);
end;

procedure TMainForm.PWMStopButtonClick(Sender: TObject);
var
  BitIndex: Longword;

begin
  BitIndex := (Sender as TButton).Tag;
  //RepCount=0 means turn off PWM.
  DO_SetPWM(CardCombo.ItemIndex, BitIndex, $80, 0);
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  WDG_WaitForEvent( CardCombo.ItemIndex );
end;

end.
